@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Modules::AdminGenericAgent;
@@ -100,13 +100,13 @@ sub Run {
 
         # get single params
         for my $Parameter (
-            qw(TicketNumber Title From To Cc Subject Body CustomerID
+            qw(TicketNumber Title MIMEBase_From MIMEBase_To MIMEBase_Cc MIMEBase_Subject MIMEBase_Body CustomerID
             CustomerUserLogin Agent SearchInArchive
             NewTitle
             NewCustomerID NewPendingTime NewPendingTimeType NewCustomerUserLogin
             NewStateID NewQueueID NewPriorityID NewOwnerID NewResponsibleID
             NewTypeID NewServiceID NewSLAID
-            NewNoteFrom NewNoteSubject NewNoteBody NewNoteTimeUnits NewModule
+            NewNoteFrom NewNoteSubject NewNoteBody NewNoteIsVisibleForCustomer NewNoteTimeUnits NewModule
             NewParamKey1 NewParamKey2 NewParamKey3 NewParamKey4
             NewParamValue1 NewParamValue2 NewParamValue3 NewParamValue4
             NewParamKey5 NewParamKey6 NewParamKey7 NewParamKey8
@@ -180,7 +180,7 @@ sub Run {
         for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
             next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
-            # extract the dynamic field value form the web request
+            # extract the dynamic field value from the web request
             my $DynamicFieldValue = $DynamicFieldBackendObject->EditFieldValueGet(
                 DynamicFieldConfig      => $DynamicFieldConfig,
                 ParamObject             => $ParamObject,
@@ -261,11 +261,11 @@ sub Run {
 
         # Check if ticket selection contains stop words
         my %StopWordsServerErrors = $Self->_StopWordsServerErrorsGet(
-            From    => $GetParam{From},
-            To      => $GetParam{To},
-            Cc      => $GetParam{Cc},
-            Subject => $GetParam{Subject},
-            Body    => $GetParam{Body},
+            MIMEBase_From    => $GetParam{MIMEBase_From},
+            MIMEBase_To      => $GetParam{MIMEBase_To},
+            MIMEBase_Cc      => $GetParam{MIMEBase_Cc},
+            MIMEBase_Subject => $GetParam{MIMEBase_Subject},
+            MIMEBase_Body    => $GetParam{MIMEBase_Body},
         );
         %Errors = ( %Errors, %StopWordsServerErrors );
 
@@ -292,9 +292,22 @@ sub Run {
             );
 
             if ($JobAddResult) {
-                return $LayoutObject->Redirect(
-                    OP => "Action=$Self->{Action}",
-                );
+
+                # if the user would like to continue editing the generic agent job, just redirect to the edit screen
+                if (
+                    defined $ParamObject->GetParam( Param => 'ContinueAfterSave' )
+                    && ( $ParamObject->GetParam( Param => 'ContinueAfterSave' ) eq '1' )
+                    )
+                {
+                    my $Profile = $Self->{Profile} || '';
+                    return $LayoutObject->Redirect( OP => "Action=$Self->{Action};Subaction=Update;Profile=$Profile" );
+                }
+                else {
+
+                    # otherwise return to overview
+                    return $LayoutObject->Redirect( OP => "Action=$Self->{Action}" );
+                }
+
             }
             else {
                 $Errors{ProfileInvalid}    = 'ServerError';
@@ -313,7 +326,9 @@ sub Run {
         );
 
         # generate search mask
-        my $Output = $LayoutObject->Header( Title => 'Edit' );
+        my $Output = $LayoutObject->Header(
+            Title => Translatable('Edit'),
+        );
         $Output .= $LayoutObject->NavigationBar();
         $Output .= $LayoutObject->Output(
             TemplateFile => 'AdminGenericAgent',
@@ -331,7 +346,9 @@ sub Run {
         $JobDataReference = $Self->_MaskUpdate(%Param);
 
         # generate search mask
-        my $Output = $LayoutObject->Header( Title => 'Edit' );
+        my $Output = $LayoutObject->Header(
+            Title => Translatable('Edit'),
+        );
 
         $Output .= $LayoutObject->NavigationBar();
         $Output .= $LayoutObject->Output(
@@ -367,9 +384,13 @@ sub Run {
     $LayoutObject->Block(
         Name => 'ActionAdd',
     );
+    $LayoutObject->Block(
+        Name => 'Filter',
+    );
     $LayoutObject->Block(
         Name => 'Overview',
     );
+
     my %Jobs = $GenericAgentObject->JobList();
 
     # if there are any data, it is shown
@@ -420,7 +441,8 @@ sub _MaskUpdate {
             Name => $Self->{Profile},
         );
     }
-    $JobData{Profile} = $Self->{Profile};
+    $JobData{Profile}   = $Self->{Profile};
+    $JobData{Subaction} = $Self->{Subaction};
 
     # get config object
     my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
@@ -745,11 +767,11 @@ sub _MaskUpdate {
     my %StopWordsServerErrors;
     if ( !$Param{StopWordsAlreadyChecked} ) {
         %StopWordsServerErrors = $Self->_StopWordsServerErrorsGet(
-            From    => $JobData{From},
-            To      => $JobData{To},
-            Cc      => $JobData{Cc},
-            Subject => $JobData{Subject},
-            Body    => $JobData{Body},
+            MIMEBase_From    => $JobData{MIMEBase_From},
+            MIMEBase_To      => $JobData{MIMEBase_To},
+            MIMEBase_Cc      => $JobData{MIMEBase_Cc},
+            MIMEBase_Subject => $JobData{MIMEBase_Subject},
+            MIMEBase_Body    => $JobData{MIMEBase_Body},
         );
     }
 
@@ -1032,14 +1054,6 @@ sub _MaskUpdate {
     for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
         next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
-        # Check if field is Attachment type ( from OTRSDynamicFieldAttachment )
-        #   this field is not updatable by Generic Agent
-        my $IsAttachement = $DynamicFieldBackendObject->HasBehavior(
-            DynamicFieldConfig => $DynamicFieldConfig,
-            Behavior           => 'IsAttachement',
-        );
-        next DYNAMICFIELD if $IsAttachement;
-
         my $PossibleValuesFilter;
 
         my $IsACLReducible = $DynamicFieldBackendObject->HasBehavior(
@@ -1155,19 +1169,14 @@ sub _MaskUpdate {
 
         # refresh event list for each event type
 
-        # hide inactive event lists
-        my $EventListHidden = '';
-        if ( $Type ne $SelectedEventType ) {
-            $EventListHidden = 'Hidden';
-        }
-
         # paint each selector
         my $EventStrg = $LayoutObject->BuildSelection(
+            PossibleNone => 0,
             Data         => $RegisteredEvents{$Type} || [],
             Name         => $Type . 'Event',
             Sort         => 'AlphanumericValue',
             PossibleNone => 0,
-            Class        => 'EventList GenericInterfaceSpacing ' . $EventListHidden,
+            Class        => 'Modernize EventList GenericInterfaceSpacing',
             Title        => $LayoutObject->{LanguageObject}->Translate('Event'),
         );
 
@@ -1188,7 +1197,7 @@ sub _MaskUpdate {
         Sort          => 'AlphanumericValue',
         SelectedValue => $SelectedEventType,
         PossibleNone  => 0,
-        Class         => '',
+        Class         => 'Modernize',
         Title         => $LayoutObject->{LanguageObject}->Translate('Type'),
     );
     $LayoutObject->Block(
@@ -1224,6 +1233,8 @@ sub _MaskRun {
         );
     }
     $JobData{Profile} = $Self->{Profile};
+    $Param{Subaction} = $Self->{Subaction};
+    $Param{Profile}   = $Self->{Profile};
 
     # dynamic fields search parameters for ticket search
     my %DynamicFieldSearchParameters;
@@ -1273,9 +1284,18 @@ sub _MaskRun {
         }
     }
 
+    # remove residual dynamic field data from job definition
+    # they are passed through dedicated variable anyway
+    PARAM_NAME:
+    for my $ParamName ( sort keys %JobData ) {
+        next PARAM_NAME if !( $ParamName =~ /^DynamicField_/ );
+        delete $JobData{$ParamName};
+    }
+
     # get needed objects
-    my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
-    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+    my $TicketObject  = $Kernel::OM->Get('Kernel::System::Ticket');
+    my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
+    my $ConfigObject  = $Kernel::OM->Get('Kernel::Config');
 
     # perform ticket search
     my $GenericAgentTicketSearch = $ConfigObject->Get("Ticket::GenericAgentTicketSearch") || {};
@@ -1333,22 +1353,26 @@ sub _MaskRun {
         );
         for my $TicketID (@TicketIDs) {
 
-            # get first article data
-            my %Data = $TicketObject->ArticleFirstArticle(
+            # Get ticket data.
+            my %Ticket = $TicketObject->TicketGet(
                 TicketID      => $TicketID,
                 DynamicFields => 0,
             );
 
-            # Fall-back for tickets without articles
-            if ( !%Data ) {
+            # Get article data.
+            my @Articles = $ArticleObject->ArticleList(
+                TicketID  => $TicketID,
+                OnlyFirst => 1,
+            );
+            my %Article;
+            for my $Article (@Articles) {
+                %Article = $ArticleObject->BackendForArticle( %{$Article} )->ArticleGet( %{$Article} );
+            }
 
-                # get ticket data instead
-                %Data = $TicketObject->TicketGet(
-                    TicketID      => $TicketID,
-                    DynamicFields => 0,
-                );
+            my %Data = ( %Ticket, %Article );
 
-                # set missing information
+            # Set missing information for tickets without articles.
+            if ( !%Article ) {
                 $Data{Subject} = $Data{Title};
             }
 
@@ -1364,7 +1388,7 @@ sub _MaskRun {
             );
             $Data{UserLastname}  = $UserInfo{UserLastname};
             $Data{UserFirstname} = $UserInfo{UserFirstname};
-
+            $Data{UserFullname}  = $UserInfo{UserFullname};
             $LayoutObject->Block(
                 Name => 'Ticket',
                 Data => \%Data,
@@ -1402,8 +1426,10 @@ sub _StopWordsServerErrorsGet {
         );
     }
 
+    my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
+
     my %StopWordsServerErrors;
-    if ( !$Kernel::OM->Get('Kernel::System::Ticket')->SearchStringStopWordsUsageWarningActive() ) {
+    if ( !$ArticleObject->SearchStringStopWordsUsageWarningActive() ) {
         return %StopWordsServerErrors;
     }
 
@@ -1419,8 +1445,8 @@ sub _StopWordsServerErrorsGet {
 
     if (%SearchStrings) {
 
-        my $StopWords = $Kernel::OM->Get('Kernel::System::Ticket')->SearchStringStopWordsFind(
-            SearchStrings => \%SearchStrings
+        my $StopWords = $ArticleObject->SearchStringStopWordsFind(
+            SearchStrings => \%SearchStrings,
         );
 
         FIELD:
