@@ -951,6 +951,19 @@ sub Run {
                     Version   => $Structure{Version}->{Content},
                 },
             );
+
+            $LayoutObject->Block(
+                Name => 'IntroForm',
+                Data => {
+                    %Param,
+                    %Data,
+                    Subaction => $Self->{Subaction},
+                    Type      => 'IntroReinstallPre',
+                    Name      => $Structure{Name}->{Content},
+                    Version   => $Structure{Version}->{Content},
+                },
+            );
+
             $LayoutObject->Block(
                 Name => 'IntroCancel',
             );
@@ -1036,6 +1049,19 @@ sub Run {
                     Version   => $Version,
                 },
             );
+
+            $LayoutObject->Block(
+                Name => 'IntroForm',
+                Data => {
+                    %Param,
+                    %Data,
+                    Subaction => '',
+                    Type      => 'IntroReinstallPost',
+                    Name      => $Name,
+                    Version   => $Version,
+                },
+            );
+
             my $Output = $LayoutObject->Header();
             $Output .= $LayoutObject->NavigationBar();
             $Output .= $LayoutObject->Output(
@@ -1099,6 +1125,19 @@ sub Run {
                     Version   => $Structure{Version}->{Content},
                 },
             );
+
+            $LayoutObject->Block(
+                Name => 'IntroForm',
+                Data => {
+                    %Param,
+                    %Data,
+                    Subaction => $Self->{Subaction},
+                    Type      => 'IntroUninstallPre',
+                    Name      => $Structure{Name}->{Content},
+                    Version   => $Structure{Version}->{Content},
+                },
+            );
+
             $LayoutObject->Block(
                 Name => 'IntroCancel',
             );
@@ -1186,6 +1225,19 @@ sub Run {
                     Version   => $Version,
                 },
             );
+
+            $LayoutObject->Block(
+                Name => 'IntroForm',
+                Data => {
+                    %Param,
+                    %Data,
+                    Subaction => '',
+                    Type      => 'IntroUninstallPost',
+                    Name      => $Name,
+                    Version   => $Version,
+                },
+            );
+
             my $Output = $LayoutObject->Header();
             $Output .= $LayoutObject->NavigationBar();
             $Output .= $LayoutObject->Output(
@@ -1373,12 +1425,14 @@ sub Run {
                 Updated        => $LayoutObject->{LanguageObject}->Translate('Updated'),
                 AlreadyUpdated => $LayoutObject->{LanguageObject}->Translate('Already up-to-date'),
                 Installed      => $LayoutObject->{LanguageObject}->Translate('Installed'),
+                Undeployed     => $LayoutObject->{LanguageObject}->Translate('Not correctly deployed'),
                 Failed         => $LayoutObject->{LanguageObject}->Translate('Failed'),
             );
             my %StatusMessages = (
                 Updated        => $LayoutObject->{LanguageObject}->Translate('Package updated correctly'),
                 AlreadyUpdated => $LayoutObject->{LanguageObject}->Translate('Package was already updated'),
                 Installed      => $LayoutObject->{LanguageObject}->Translate('Dependency installed correctly'),
+                Undeployed     => $LayoutObject->{LanguageObject}->Translate('The package needs to be reinstalled'),
                 Cyclic       => $LayoutObject->{LanguageObject}->Translate('The package contains cyclic dependencies'),
                 NotFound     => $LayoutObject->{LanguageObject}->Translate('Not found in on-line repositories'),
                 WrongVersion => $LayoutObject->{LanguageObject}->Translate('Required version is higher than available'),
@@ -1388,14 +1442,18 @@ sub Run {
             );
 
             if ( IsHashRefWithData($UpgradeResult) ) {
-                for my $StatusType (qw(Updated Installed AlreadyUpdated)) {
+                for my $StatusType (qw(Updated Installed AlreadyUpdated Undeployed)) {
                     for my $PackageName ( sort keys %{ $UpgradeResult->{$StatusType} } ) {
+                        my $Class = 'Success';
+                        if ( $StatusType eq 'Installed' || $StatusType eq 'Undeployed' ) {
+                            $Class = 'Warning';
+                        }
                         $PackageList{$PackageName} = {
                             Name          => $PackageName,
                             Status        => $StatusType,
                             StatusDisplay => $StatusStings{$StatusType},
                             StatusMessage => $StatusMessages{$StatusType},
-                            Class         => $StatusType eq 'Installed' ? 'Warning' : 'Success',
+                            Class         => $Class,
                         };
                     }
                 }
@@ -1814,7 +1872,7 @@ sub Run {
         );
         $TargetDateTimeObject->Add( Days => 1 );
         if ( $CurrentDateTimeObject > $TargetDateTimeObject ) {
-            $PackageObject->PackageUpgradeAllDataDelete()
+            $PackageObject->PackageUpgradeAllDataDelete();
         }
     }
 
@@ -2034,7 +2092,8 @@ sub _InstallHandling {
     # translate description
     if ( $LayoutObject->{LanguageObject} ) {
         $VerifyInfo{Description} = $LayoutObject->{LanguageObject}->Translate(
-            $VerifyInfo{Description}
+            $VerifyInfo{Description},
+            $VerifyInfo{PackageInstallPossible} ? '' : $LayoutObject->{Baselink},
         );
     }
 
@@ -2053,9 +2112,24 @@ sub _InstallHandling {
             },
         );
 
-        $LayoutObject->Block(
-            Name => 'IntroCancel',
-        );
+        if ( $VerifyInfo{PackageInstallPossible} ) {
+
+            $LayoutObject->Block(
+                Name => 'IntroForm',
+                Data => {
+                    %Param,
+                    %VerifyInfo,
+                    Subaction => $Self->{Subaction},
+                    Type      => 'IntroInstallVendor',
+                    Name      => $Structure{Name}->{Content},
+                    Version   => $Structure{Version}->{Content},
+                },
+            );
+
+            $LayoutObject->Block(
+                Name => 'IntroCancel',
+            );
+        }
 
         my $Output = $LayoutObject->Header();
         $Output .= $LayoutObject->NavigationBar();
@@ -2140,6 +2214,18 @@ sub _InstallHandling {
             );
         }
 
+        $LayoutObject->Block(
+            Name => 'IntroForm',
+            Data => {
+                %Param,
+                %Data,
+                Subaction => $Self->{Subaction},
+                Type      => 'IntroInstallPre',
+                Name      => $Structure{Name}->{Content},
+                Version   => $Structure{Version}->{Content},
+            },
+        );
+
         $LayoutObject->Block(
             Name => 'IntroCancel',
         );
@@ -2183,6 +2269,18 @@ sub _InstallHandling {
                 },
             );
 
+            $LayoutObject->Block(
+                Name => 'IntroForm',
+                Data => {
+                    %Param,
+                    %Data,
+                    Subaction => 'Install',
+                    Type      => 'IntroInstallPost',
+                    Name      => $Structure{Name}->{Content},
+                    Version   => $Structure{Version}->{Content},
+                },
+            );
+
             if ( $Verified eq 'verified' ) {
                 $LayoutObject->Block(
                     Name => 'OTRSVerifyLogo',
@@ -2284,6 +2382,19 @@ sub _UpgradeHandling {
                 Version   => $Structure{Version}->{Content},
             },
         );
+
+        $LayoutObject->Block(
+            Name => 'IntroForm',
+            Data => {
+                %Param,
+                %Data,
+                Subaction => $Self->{Subaction},
+                Type      => 'IntroUpgradePre',
+                Name      => $Structure{Name}->{Content},
+                Version   => $Structure{Version}->{Content},
+            },
+        );
+
         $LayoutObject->Block(
             Name => 'IntroCancel',
         );
@@ -2320,6 +2431,19 @@ sub _UpgradeHandling {
                     Version   => $Structure{Version}->{Content},
                 },
             );
+
+            $LayoutObject->Block(
+                Name => 'IntroForm',
+                Data => {
+                    %Param,
+                    %Data,
+                    Subaction => '',
+                    Type      => 'IntroUpgradePost',
+                    Name      => $Structure{Name}->{Content},
+                    Version   => $Structure{Version}->{Content},
+                },
+            );
+
             my $Output = $LayoutObject->Header();
             $Output .= $LayoutObject->NavigationBar();
             $Output .= $LayoutObject->Output(
