@@ -113,12 +113,12 @@ sub Run {
         else {
             my @MergeStateList = $StateObject->StateGetStatesByType(
                 StateType => ['merged'],
-                Result    => 'Name',
+                Result    => 'ID',
             );
 
             if (
                 scalar @MergeStateList == 1
-                && $MergeStateList[0] eq $GetParam{Name}
+                && $MergeStateList[0] eq $GetParam{ID}
                 && $Kernel::OM->Get('Kernel::System::Valid')->ValidLookup( ValidID => $GetParam{ValidID} ) ne 'valid'
                 )
             {
@@ -252,7 +252,7 @@ sub Run {
 
         my @MergeStateList = $StateObject->StateGetStatesByType(
             StateType => ['merged'],
-            Result    => 'Name',
+            Result    => 'ID',
         );
         if (
             !@MergeStateList
