@@ -37,12 +37,11 @@ sub Run {
     $Self->{CurrentUserID} = $Self->{UserID};
     if (
         $EditUserID
-        && $EditUserID != $Self->{UserID}
         && $Self->_CheckEditPreferencesPermission()
         )
     {
         $Self->{CurrentUserID}       = $EditUserID;
-        $Self->{CurrentUserIDNotice} = 1;
+        $Self->{EditingAnotherAgent} = 1;
     }
 
     # ------------------------------------------------------------ #
@@ -485,14 +484,41 @@ sub Run {
         return $Output;
     }
 
+    # ------------------------------------------------------------ #
+    # Return user favourite system configuration settings.
+    # ------------------------------------------------------------ #
+    elsif ( $Self->{Subaction} eq 'UserSystemConfigurationFavourites' ) {
+
+        my $Favourites      = [];
+        my %UserPreferences = $Kernel::OM->Get('Kernel::System::User')->GetPreferences(
+            UserID => $Self->{UserID},
+        );
+
+        if ( $UserPreferences{UserSystemConfigurationFavourites} ) {
+            $Favourites = $Kernel::OM->Get('Kernel::System::JSON')
+                ->Decode( Data => $UserPreferences{UserSystemConfigurationFavourites} );
+        }
+
+        my $JSON = $Kernel::OM->Get('Kernel::System::JSON')->Encode(
+            Data => $Favourites,
+        );
+
+        return $LayoutObject->Attachment(
+            ContentType => 'application/json; charset=' . $LayoutObject->{Charset},
+            Content     => $JSON,
+            Type        => 'inline',
+            NoCache     => 1,
+        );
+    }
+
     # ------------------------------------------------------------ #
     # show AJAX navigation tree
     # ------------------------------------------------------------ #
     if ( $Self->{Subaction} eq 'AJAXNavigationTree' ) {
 
-        my $Category = $ParamObject->GetParam( Param => 'Category' ) || '';
+        my $Category               = $ParamObject->GetParam( Param => 'Category' )               || '';
         my $UserModificationActive = $ParamObject->GetParam( Param => 'UserModificationActive' ) || '0';
-        my $IsValid = $ParamObject->GetParam( Param => 'IsValid' ) // undef;
+        my $IsValid                = $ParamObject->GetParam( Param => 'IsValid' ) // undef;
 
         my %Tree = $Kernel::OM->Get('Kernel::System::SysConfig')->ConfigurationNavigationTree(
             Action                 => 'AgentPreferences',
@@ -540,7 +566,7 @@ sub Run {
             TemplateFile => 'AgentPreferencesOverview',
             Data         => {
                 Items               => \@PreferencesGroups,
-                CurrentUserIDNotice => $Self->{CurrentUserIDNotice},
+                EditingAnotherAgent => $Self->{EditingAnotherAgent},
                 CurrentUserFullname => $UserObject->UserName( UserID => $Self->{CurrentUserID} ),
                 CurrentUserID       => $Self->{CurrentUserID},
                 View                => $UserPreferences{AgentPreferencesView} || 'Grid',
@@ -572,12 +598,11 @@ sub AgentPreferencesForm {
     $Self->{CurrentUserID} = $Self->{UserID};
     if (
         $EditUserID
-        && $EditUserID != $Self->{UserID}
         && $Self->_CheckEditPreferencesPermission()
         )
     {
+        $Self->{EditingAnotherAgent} = 1;
         $Self->{CurrentUserID}       = $EditUserID;
-        $Self->{CurrentUserIDNotice} = 1;
     }
 
     # Show navigation in advanced group
@@ -638,7 +663,7 @@ sub AgentPreferencesForm {
             %Param,
             CategoriesStrg      => $Self->_GetCategoriesStrg(),
             RootNavigation      => $RootNavigation,
-            CurrentUserIDNotice => $Self->{CurrentUserIDNotice},
+            EditingAnotherAgent => $Self->{EditingAnotherAgent},
             CurrentUserFullname =>
                 $Kernel::OM->Get('Kernel::System::User')->UserName( UserID => $Self->{CurrentUserID} ),
             CurrentUserID => $Self->{CurrentUserID},
@@ -689,7 +714,10 @@ sub AgentPreferencesForm {
         next PRIO if !$ConfigObject->{PreferencesGroups}->{$Group};
 
         my %Preference = %{ $ConfigObject->{PreferencesGroups}->{$Group} };
-        next PRIO if !$Preference{Active};
+
+        if ( !$Self->{EditingAnotherAgent} && !$Preference{Active} ) {
+            next PRIO;
+        }
 
         # load module
         my $Module = $Preference{Module} || 'Kernel::Output::HTML::Preferences::Generic';
@@ -865,7 +893,7 @@ sub _SettingReset {
     }
     elsif ( !$SysConfigObject->can('UserSettingValueDelete') ) {    # OTRS Business Solution™
         $Result{Data}->{Error} = $LayoutObject->{LanguageObject}->Translate(
-            "This feature is part of the %s Please contact us at %s for an upgrade."
+            "This feature is part of the %s. Please contact us at %s for an upgrade."
             , 'OTRS Business Solution™'
             , 'sales@otrs.com'
         );
