@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Modules::AgentTicketMerge;
@@ -11,17 +11,15 @@ package Kernel::Modules::AgentTicketMerge;
 use strict;
 use warnings;
 
-use Mail::Address;
-
-use Kernel::Language qw(Translatable);
 use Kernel::System::VariableCheck qw(:all);
+use Kernel::Language qw(Translatable);
+use Mail::Address;
 
 our $ObjectManagerDisabled = 1;
 
 sub new {
     my ( $Type, %Param ) = @_;
 
-    # allocate new hash for object
     my $Self = {%Param};
     bless( $Self, $Type );
 
@@ -35,10 +33,8 @@ sub Run {
     my %Error;
     my %GetParam;
 
-    # get needed objects
     my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
-    # check needed stuff
     if ( !$Self->{TicketID} ) {
         return $LayoutObject->ErrorScreen(
             Message => Translatable('No TicketID is given!'),
@@ -113,7 +109,7 @@ sub Run {
                     NewUserID => $Self->{UserID},
                 );
 
-                # show lock state
+                # Show lock state.
                 if ($Success) {
                     $LayoutObject->Block(
                         Name => 'PropertiesLock',
@@ -170,8 +166,8 @@ sub Run {
 
         # get all parameters
         for my $Parameter (qw( From To Subject Body InformSender MainTicketNumber )) {
-            $GetParam{$Parameter}
-                = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => $Parameter ) || '';
+            $GetParam{$Parameter} = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => $Parameter )
+                || '';
         }
 
         # rewrap body if no rich text is used
@@ -258,8 +254,8 @@ sub Run {
                 $Param{RichTextHeight} = $Config->{RichTextHeight} || 0;
                 $Param{RichTextWidth}  = $Config->{RichTextWidth}  || 0;
 
-                $LayoutObject->Block(
-                    Name => 'RichText',
+                # set up rich text editor
+                $LayoutObject->SetRichTextParameters(
                     Data => \%Param,
                 );
 
@@ -305,7 +301,7 @@ sub Run {
             || $TicketMergeResult eq 'NoValidMergeStates'
             )
         {
-            my $Output .= $LayoutObject->Header(
+            my $Output = $LayoutObject->Header(
                 Type      => 'Small',
                 BodyClass => 'Popup',
             );
@@ -324,11 +320,10 @@ sub Run {
                 $Param{RichTextHeight} = $Config->{RichTextHeight} || 0;
                 $Param{RichTextWidth}  = $Config->{RichTextWidth}  || 0;
 
-                $LayoutObject->Block(
-                    Name => 'RichText',
+                # set up rich text editor
+                $LayoutObject->SetRichTextParameters(
                     Data => \%Param,
                 );
-
             }
 
             $Output .= $LayoutObject->Output(
@@ -357,20 +352,25 @@ sub Run {
                 $GetParam{Body} =~ s/(&lt;|<)OTRS_TICKET(&gt;|>)/$Ticket{TicketNumber}/g;
                 $GetParam{Body}
                     =~ s/(&lt;|<)OTRS_MERGE_TO_TICKET(&gt;|>)/$GetParam{'MainTicketNumber'}/g;
-                my $ArticleID = $TicketObject->ArticleSend(
-                    ArticleType    => 'email-external',
-                    SenderType     => 'agent',
-                    TicketID       => $Self->{TicketID},
-                    HistoryType    => 'SendAnswer',
-                    HistoryComment => "Merge info to '$GetParam{To}'.",
-                    From           => $GetParam{From},
-                    Email          => $GetParam{Email},
-                    To             => $GetParam{To},
-                    Subject        => $GetParam{Subject},
-                    UserID         => $Self->{UserID},
-                    Body           => $GetParam{Body},
-                    Charset        => $LayoutObject->{UserCharset},
-                    MimeType       => $MimeType,
+
+                my $EmailArticleBackendObject = $Kernel::OM->Get('Kernel::System::Ticket::Article')->BackendForChannel(
+                    ChannelName => 'Email',
+                );
+
+                my $ArticleID = $EmailArticleBackendObject->ArticleSend(
+                    TicketID             => $Self->{TicketID},
+                    SenderType           => 'agent',
+                    IsVisibleForCustomer => 1,
+                    HistoryType          => 'SendAnswer',
+                    HistoryComment       => "Merge info to '$GetParam{To}'.",
+                    From                 => $GetParam{From},
+                    Email                => $GetParam{Email},
+                    To                   => $GetParam{To},
+                    Subject              => $GetParam{Subject},
+                    UserID               => $Self->{UserID},
+                    Body                 => $GetParam{Body},
+                    Charset              => $LayoutObject->{UserCharset},
+                    MimeType             => $MimeType,
                 );
                 if ( !$ArticleID ) {
 
@@ -386,13 +386,40 @@ sub Run {
         }
     }
     else {
+        my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
 
-        # get last article
-        my %Article = $TicketObject->ArticleLastCustomerArticle(
-            TicketID      => $Self->{TicketID},
-            DynamicFields => 1,
+        # Get last customer article.
+        my @Articles = $ArticleObject->ArticleList(
+            TicketID   => $Self->{TicketID},
+            SenderType => 'customer',
+            OnlyLast   => 1,
         );
 
+        # If the ticket has no customer article, get the last agent article.
+        if ( !@Articles ) {
+            @Articles = $ArticleObject->ArticleList(
+                TicketID   => $Self->{TicketID},
+                SenderType => 'agent',
+                OnlyLast   => 1,
+            );
+        }
+
+        # Finally, if everything failed, get latest article.
+        if ( !@Articles ) {
+            @Articles = $ArticleObject->ArticleList(
+                TicketID => $Self->{TicketID},
+                OnlyLast => 1,
+            );
+        }
+
+        my %Article;
+        for my $Article (@Articles) {
+            %Article = $ArticleObject->BackendForArticle( %{$Article} )->ArticleGet(
+                %{$Article},
+                DynamicFields => 1,
+            );
+        }
+
         # merge box
         my $Output = $LayoutObject->Header(
             Value     => $Ticket{TicketNumber},
@@ -419,7 +446,7 @@ sub Run {
 
         # prepare subject ...
         $Article{Subject} = $TicketObject->TicketSubjectBuild(
-            TicketNumber => $Article{TicketNumber},
+            TicketNumber => $Ticket{TicketNumber},
             Subject      => $Article{Subject} || '',
         );
 
@@ -454,8 +481,8 @@ sub Run {
             $Param{RichTextHeight} = $Config->{RichTextHeight} || 0;
             $Param{RichTextWidth}  = $Config->{RichTextWidth}  || 0;
 
-            $LayoutObject->Block(
-                Name => 'RichText',
+            # set up rich text editor
+            $LayoutObject->SetRichTextParameters(
                 Data => \%Param,
             );
         }
