@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -362,7 +362,9 @@ sub Run {
                 );
 
                 if ( !IsHashRefWithData($ValidationResult) ) {
-                    my $Output = $LayoutObject->CustomerHeader( Title => 'Error' );
+                    my $Output = $LayoutObject->CustomerHeader(
+                        Title => Translatable('Error'),
+                    );
                     $Output .= $LayoutObject->CustomerError(
                         Message =>
                             $LayoutObject->{LanguageObject}
@@ -440,7 +442,7 @@ sub Run {
 
             # set the correct queue name in $To if it was altered
             if ( $To ne $Tos->{$NewQueueID} ) {
-                $To = $Tos->{$NewQueueID}
+                $To = $Tos->{$NewQueueID};
             }
         }
 
@@ -596,7 +598,9 @@ sub Run {
         );
 
         if ( !$ArticleID ) {
-            my $Output = $LayoutObject->CustomerHeader( Title => 'Error' );
+            my $Output = $LayoutObject->CustomerHeader(
+                Title => Translatable('Error'),
+            );
             $Output .= $LayoutObject->CustomerError();
             $Output .= $LayoutObject->CustomerFooter();
             return $Output;
