@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Auth::DB;
@@ -140,11 +140,11 @@ sub Auth {
 
             if ( $Magic eq '$apr1$' ) {
                 $CryptedPw = apache_md5_crypt( $Pw, $Salt );
-                $Method    = 'apache_md5_crypt';
+                $Method = 'apache_md5_crypt';
             }
             else {
                 $CryptedPw = unix_md5_crypt( $Pw, $Salt );
-                $Method    = 'unix_md5_crypt';
+                $Method = 'unix_md5_crypt';
             }
 
         }
@@ -153,10 +153,17 @@ sub Auth {
         elsif ( $GetPw =~ m{\A [0-9a-f]{64} \z}xmsi ) {
 
             my $SHAObject = Digest::SHA->new('sha256');
-
-            # encode output, needed by sha256_hex() only non utf8 signs
             $EncodeObject->EncodeOutput( \$Pw );
+            $SHAObject->add($Pw);
+            $CryptedPw = $SHAObject->hexdigest();
+            $Method    = 'sha256';
+        }
 
+        # sha512 pw
+        elsif ( $GetPw =~ m{\A [0-9a-f]{128} \z}xmsi ) {
+
+            my $SHAObject = Digest::SHA->new('sha512');
+            $EncodeObject->EncodeOutput( \$Pw );
             $SHAObject->add($Pw);
             $CryptedPw = $SHAObject->hexdigest();
             $Method    = 'sha256';
@@ -232,7 +239,7 @@ sub Auth {
         $EncodeObject->EncodeOutput( \$Pw );
         $EncodeObject->EncodeOutput( \$Salt );
         $CryptedPw = crypt( $Pw, $Salt );
-        $Method    = 'crypt';
+        $Method = 'crypt';
     }
 
     # just in case for debug!
