@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -266,7 +266,7 @@ sub _Fetch {
         $NumberOfMessages = scalar @{$Messages};
 
         if ($CMD) {
-            print "$AuthType: I found $NumberOfMessages messages on $Param{Login}/$Param{Host}. "
+            print "$AuthType: I found $NumberOfMessages messages on $Param{Login}/$Param{Host}. ";
         }
 
         return 1;
