@@ -1,6 +1,5 @@
 # --
-# Kernel/System/State.pm - All ticket state related functions
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -12,62 +11,30 @@ package Kernel::System::State;
 use strict;
 use warnings;
 
-use Kernel::System::CacheInternal;
-use Kernel::System::SysConfig;
-use Kernel::System::Time;
-use Kernel::System::Valid;
+our @ObjectDependencies = (
+    'Kernel::Config',
+    'Kernel::System::Cache',
+    'Kernel::System::DB',
+    'Kernel::System::Log',
+    'Kernel::System::SysConfig',
+    'Kernel::System::Valid',
+);
 
 =head1 NAME
 
 Kernel::System::State - state lib
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 All ticket state functions.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
-
-=cut
-
-=item new()
+=head2 new()
 
 create an object
 
-    use Kernel::Config;
-    use Kernel::System::Encode;
-    use Kernel::System::Log;
-    use Kernel::System::Main;
-    use Kernel::System::DB;
-    use Kernel::System::State;
-
-    my $ConfigObject = Kernel::Config->new();
-    my $EncodeObject = Kernel::System::Encode->new(
-        ConfigObject => $ConfigObject,
-    );
-    my $LogObject = Kernel::System::Log->new(
-        ConfigObject => $ConfigObject,
-        EncodeObject => $EncodeObject,
-    );
-    my $MainObject = Kernel::System::Main->new(
-        ConfigObject => $ConfigObject,
-        EncodeObject => $EncodeObject,
-        LogObject    => $LogObject,
-    );
-    my $DBObject = Kernel::System::DB->new(
-        ConfigObject => $ConfigObject,
-        EncodeObject => $EncodeObject,
-        LogObject    => $LogObject,
-        MainObject   => $MainObject,
-    );
-    my $StateObject = Kernel::System::State->new(
-        ConfigObject => $ConfigObject,
-        LogObject    => $LogObject,
-        DBObject     => $DBObject,
-        MainObject   => $MainObject,
-        EncodeObject => $EncodeObject,
-    );
+    my $StateObject = $Kernel::OM->Get('Kernel::System::State');
 
 =cut
 
@@ -78,28 +45,13 @@ sub new {
     my $Self = {};
     bless( $Self, $Type );
 
-    # check needed objects
-    for (qw(DBObject ConfigObject LogObject MainObject EncodeObject)) {
-        $Self->{$_} = $Param{$_} || die "Got no $_!";
-    }
-
-    # create additional objects
-    $Self->{ValidObject}         = Kernel::System::Valid->new( %{$Self} );
-    $Self->{CacheInternalObject} = Kernel::System::CacheInternal->new(
-        %Param,
-        Type => 'State',
-        TTL  => 60 * 60 * 24 * 20,
-    );
-
-    # check needed config options
-    for (qw(Ticket::ViewableStateType Ticket::UnlockStateType)) {
-        $Self->{ConfigObject}->Get($_) || die "Need $_ in Kernel/Config.pm!\n";
-    }
+    $Self->{CacheType} = 'State';
+    $Self->{CacheTTL}  = 60 * 60 * 24 * 20;
 
     return $Self;
 }
 
-=item StateAdd()
+=head2 StateAdd()
 
 add new states
 
@@ -119,7 +71,7 @@ sub StateAdd {
     # check needed stuff
     for (qw(Name ValidID TypeID UserID)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
@@ -127,8 +79,11 @@ sub StateAdd {
         }
     }
 
+    # get database object
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
+
     # store data
-    return if !$Self->{DBObject}->Do(
+    return if !$DBObject->Do(
         SQL => 'INSERT INTO ticket_state (name, valid_id, type_id, comments,'
             . ' create_time, create_by, change_time, change_by)'
             . ' VALUES (?, ?, ?, ?, current_timestamp, ?, current_timestamp, ?)',
@@ -139,7 +94,7 @@ sub StateAdd {
     );
 
     # get new state id
-    return if !$Self->{DBObject}->Prepare(
+    return if !$DBObject->Prepare(
         SQL   => 'SELECT id FROM ticket_state WHERE name = ?',
         Bind  => [ \$Param{Name} ],
         Limit => 1,
@@ -147,19 +102,21 @@ sub StateAdd {
 
     # fetch the result
     my $ID;
-    while ( my @Row = $Self->{DBObject}->FetchrowArray() ) {
+    while ( my @Row = $DBObject->FetchrowArray() ) {
         $ID = $Row[0];
     }
 
     return if !$ID;
 
     # delete cache
-    $Self->{CacheInternalObject}->CleanUp();
+    $Kernel::OM->Get('Kernel::System::Cache')->CleanUp(
+        Type => $Self->{CacheType},
+    );
 
     return $ID;
 }
 
-=item StateGet()
+=head2 StateGet()
 
 get state attributes
 
@@ -191,7 +148,7 @@ sub StateGet {
 
     # check needed stuff
     if ( !$Param{ID} && !$Param{Name} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => "Need ID or Name!"
         );
@@ -206,9 +163,15 @@ sub StateGet {
     else {
         $CacheKey = 'StateGet::ID::' . $Param{ID};
     }
-    my $Cache = $Self->{CacheInternalObject}->Get( Key => $CacheKey );
+    my $Cache = $Kernel::OM->Get('Kernel::System::Cache')->Get(
+        Type => $Self->{CacheType},
+        Key  => $CacheKey,
+    );
     return %{$Cache} if $Cache;
 
+    # get database object
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
+
     # sql
     my @Bind;
     my $SQL = 'SELECT ts.id, ts.name, ts.valid_id, ts.comments, ts.type_id, tst.name,'
@@ -223,7 +186,7 @@ sub StateGet {
         push @Bind, \$Param{ID};
     }
 
-    return if !$Self->{DBObject}->Prepare(
+    return if !$DBObject->Prepare(
         SQL   => $SQL,
         Bind  => \@Bind,
         Limit => 1,
@@ -231,7 +194,7 @@ sub StateGet {
 
     # fetch the result
     my %Data;
-    while ( my @Data = $Self->{DBObject}->FetchrowArray() ) {
+    while ( my @Data = $DBObject->FetchrowArray() ) {
         %Data = (
             ID         => $Data[0],
             Name       => $Data[1],
@@ -245,14 +208,16 @@ sub StateGet {
     }
 
     # set cache
-    $Self->{CacheInternalObject}->Set(
+    $Kernel::OM->Get('Kernel::System::Cache')->Set(
+        Type  => $Self->{CacheType},
+        TTL   => $Self->{CacheTTL},
         Key   => $CacheKey,
         Value => \%Data,
     );
 
     # no data found...
     if ( !%Data ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => "State '$Param{Name}' not found!",
         );
@@ -262,7 +227,7 @@ sub StateGet {
     return %Data;
 }
 
-=item StateUpdate()
+=head2 StateUpdate()
 
 update state attributes
 
@@ -272,7 +237,6 @@ update state attributes
         Comment        => 'some comment',
         ValidID        => 1,
         TypeID         => 1,
-        CheckSysConfig => 0,   # (optional) default 1
         UserID         => 123,
     );
 
@@ -284,7 +248,7 @@ sub StateUpdate {
     # check needed stuff
     for (qw(ID Name ValidID TypeID UserID)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
@@ -292,13 +256,11 @@ sub StateUpdate {
         }
     }
 
-    # check CheckSysConfig param
-    if ( !defined $Param{CheckSysConfig} ) {
-        $Param{CheckSysConfig} = 1;
-    }
+    # get database object
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
 
     # sql
-    return if !$Self->{DBObject}->Do(
+    return if !$DBObject->Do(
         SQL => 'UPDATE ticket_state SET name = ?, comments = ?, type_id = ?, '
             . ' valid_id = ?, change_time = current_timestamp, change_by = ? '
             . ' WHERE id = ?',
@@ -309,27 +271,14 @@ sub StateUpdate {
     );
 
     # delete cache
-    $Self->{CacheInternalObject}->CleanUp();
-
-    # check all sysconfig options
-    return 1 if !$Param{CheckSysConfig};
-
-    # create a time object locally, needed for the local SysConfigObject
-    my $TimeObject = Kernel::System::Time->new( %{$Self} );
-
-    # create a sysconfig object locally for performance reasons
-    my $SysConfigObject = Kernel::System::SysConfig->new(
-        %{$Self},
-        TimeObject => $TimeObject,
+    $Kernel::OM->Get('Kernel::System::Cache')->CleanUp(
+        Type => $Self->{CacheType},
     );
 
-    # check all sysconfig options and correct them automatically if neccessary
-    $SysConfigObject->ConfigItemCheckAll();
-
     return 1;
 }
 
-=item StateGetStatesByType()
+=head2 StateGetStatesByType()
 
 get list of states for a type or a list of state types.
 
@@ -356,7 +305,7 @@ sub StateGetStatesByType {
 
     # check needed stuff
     if ( !$Param{Result} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Result!'
         );
@@ -364,7 +313,7 @@ sub StateGetStatesByType {
     }
 
     if ( !$Param{Type} && !$Param{StateType} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Type or StateType!'
         );
@@ -389,7 +338,10 @@ sub StateGetStatesByType {
     }
 
     # check cache
-    my $Cache = $Self->{CacheInternalObject}->Get( Key => $CacheKey );
+    my $Cache = $Kernel::OM->Get('Kernel::System::Cache')->Get(
+        Type => $Self->{CacheType},
+        Key  => $CacheKey,
+    );
     if ($Cache) {
         if ( $Param{Result} eq 'Name' ) {
             return @{ $Cache->{Name} };
@@ -406,11 +358,15 @@ sub StateGetStatesByType {
     my @ID;
     my %Data;
     if ( $Param{Type} ) {
-        if ( $Self->{ConfigObject}->Get( 'Ticket::' . $Param{Type} . 'StateType' ) ) {
-            @StateType = @{ $Self->{ConfigObject}->Get( 'Ticket::' . $Param{Type} . 'StateType' ) };
+
+        # get config object
+        my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
+        if ( $ConfigObject->Get( 'Ticket::' . $Param{Type} . 'StateType' ) ) {
+            @StateType = @{ $ConfigObject->Get( 'Ticket::' . $Param{Type} . 'StateType' ) };
         }
         else {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Type 'Ticket::$Param{Type}StateType' not found in Kernel/Config.pm!",
             );
@@ -426,19 +382,22 @@ sub StateGetStatesByType {
         }
     }
 
-    @StateType = map { $Self->{DBObject}->Quote($_) } @StateType;
+    # get database object
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
+
+    @StateType = map { $DBObject->Quote($_) } @StateType;
 
     my $SQL = ''
         . 'SELECT ts.id, ts.name, tst.name'
         . ' FROM ticket_state ts, ticket_state_type tst'
         . ' WHERE tst.id = ts.type_id'
         . " AND tst.name IN ('${\(join '\', \'', sort @StateType)}' )"
-        . " AND ts.valid_id IN ( ${\(join ', ', $Self->{ValidObject}->ValidIDsGet())} )";
+        . " AND ts.valid_id IN ( ${\(join ', ', $Kernel::OM->Get('Kernel::System::Valid')->ValidIDsGet())} )";
 
-    return if !$Self->{DBObject}->Prepare( SQL => $SQL );
+    return if !$DBObject->Prepare( SQL => $SQL );
 
     # fetch the result
-    while ( my @Data = $Self->{DBObject}->FetchrowArray() ) {
+    while ( my @Data = $DBObject->FetchrowArray() ) {
         push @Name, $Data[1];
         push @ID,   $Data[0];
         $Data{ $Data[0] } = $Data[1];
@@ -452,7 +411,9 @@ sub StateGetStatesByType {
     };
 
     # set permanent cache
-    $Self->{CacheInternalObject}->Set(
+    $Kernel::OM->Get('Kernel::System::Cache')->Set(
+        Type  => $Self->{CacheType},
+        TTL   => $Self->{CacheTTL},
         Key   => $CacheKey,
         Value => $All,
     );
@@ -467,7 +428,7 @@ sub StateGetStatesByType {
     return @ID;
 }
 
-=item StateList()
+=head2 StateList()
 
 get state list as a hash of ID, Name pairs
 
@@ -506,7 +467,7 @@ sub StateList {
 
     # check needed stuff
     if ( !$Param{UserID} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'UserID!'
         );
@@ -520,25 +481,34 @@ sub StateList {
 
     # check cache
     my $CacheKey = 'StateList::' . $Valid;
-    my $Cache = $Self->{CacheInternalObject}->Get( Key => $CacheKey );
+    my $Cache    = $Kernel::OM->Get('Kernel::System::Cache')->Get(
+        Type => $Self->{CacheType},
+        Key  => $CacheKey,
+    );
     return %{$Cache} if $Cache;
 
     # sql
     my $SQL = 'SELECT id, name FROM ticket_state';
     if ($Valid) {
-        $SQL .= " WHERE valid_id IN ( ${\(join ', ', $Self->{ValidObject}->ValidIDsGet())} )";
+        $SQL
+            .= " WHERE valid_id IN ( ${\(join ', ', $Kernel::OM->Get('Kernel::System::Valid')->ValidIDsGet())} )";
     }
 
-    return if !$Self->{DBObject}->Prepare( SQL => $SQL );
+    # get database object
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
+
+    return if !$DBObject->Prepare( SQL => $SQL );
 
     # fetch the result
     my %Data;
-    while ( my @Row = $Self->{DBObject}->FetchrowArray() ) {
+    while ( my @Row = $DBObject->FetchrowArray() ) {
         $Data{ $Row[0] } = $Row[1];
     }
 
     # set cache
-    $Self->{CacheInternalObject}->Set(
+    $Kernel::OM->Get('Kernel::System::Cache')->Set(
+        Type  => $Self->{CacheType},
+        TTL   => $Self->{CacheTTL},
         Key   => $CacheKey,
         Value => \%Data,
     );
@@ -546,7 +516,7 @@ sub StateList {
     return %Data;
 }
 
-=item StateLookup()
+=head2 StateLookup()
 
 returns the id or the name of a state
 
@@ -565,7 +535,7 @@ sub StateLookup {
 
     # check needed stuff
     if ( !$Param{State} && !$Param{StateID} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             State   => 'error',
             Message => 'Need State or StateID!'
         );
@@ -595,7 +565,7 @@ sub StateLookup {
 
     # check if data exists
     if ( !defined $ReturnData ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => "No $Key for $Value found!",
         );
@@ -605,7 +575,7 @@ sub StateLookup {
     return $ReturnData;
 }
 
-=item StateTypeList()
+=head2 StateTypeList()
 
 get state type list as a hash of ID, Name pairs
 
@@ -632,7 +602,7 @@ sub StateTypeList {
 
     # check needed stuff
     if ( !$Param{UserID} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'UserID!'
         );
@@ -641,40 +611,48 @@ sub StateTypeList {
 
     # check cache
     my $CacheKey = 'StateTypeList';
-    my $Cache = $Self->{CacheInternalObject}->Get( Key => $CacheKey );
+    my $Cache    = $Kernel::OM->Get('Kernel::System::Cache')->Get(
+        Type => $Self->{CacheType},
+        Key  => $CacheKey,
+    );
     return %{$Cache} if $Cache;
 
+    # get database object
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
+
     # sql
-    return if !$Self->{DBObject}->Prepare(
+    return if !$DBObject->Prepare(
         SQL => 'SELECT id, name FROM ticket_state_type',
     );
 
     # fetch the result
     my %Data;
-    while ( my @Row = $Self->{DBObject}->FetchrowArray() ) {
+    while ( my @Row = $DBObject->FetchrowArray() ) {
         $Data{ $Row[0] } = $Row[1];
     }
 
     # set cache
-    $Self->{CacheInternalObject}->Set(
+    $Kernel::OM->Get('Kernel::System::Cache')->Set(
+        Type  => $Self->{CacheType},
+        TTL   => $Self->{CacheTTL},
         Key   => $CacheKey,
-        Value => \%Data
+        Value => \%Data,
     );
 
     return %Data;
 }
 
-=item StateTypeLookup()
+=head2 StateTypeLookup()
 
 returns the id or the name of a state type
 
-    my $StateTypeID = $StateTypeObject->StateTypeLookup(
+    my $StateTypeID = $StateObject->StateTypeLookup(
         StateType => 'pending auto',
     );
 
 or
 
-    my $StateType = $StateTypeObject->StateTypeLookup(
+    my $StateType = $StateObject->StateTypeLookup(
         StateTypeID => 1,
     );
 
@@ -685,7 +663,7 @@ sub StateTypeLookup {
 
     # check needed stuff
     if ( !$Param{StateType} && !$Param{StateTypeID} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             StateType => 'error',
             Message   => 'Need StateType or StateTypeID!',
         );
@@ -714,7 +692,7 @@ sub StateTypeLookup {
 
     # check if data exists
     if ( !defined $ReturnData ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => "No $Key for $Value found!",
         );
@@ -726,8 +704,6 @@ sub StateTypeLookup {
 
 1;
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
 This software is part of the OTRS project (L<http://otrs.org/>).
