@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Ticket::ColumnFilter;
@@ -24,22 +24,16 @@ our @ObjectDependencies = (
 
 Kernel::System::Ticket::ColumnFilter - Column Filter library
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 All functions for Column Filters.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
+=head2 new()
 
-=cut
+Don't use the constructor directly, use the ObjectManager instead:
 
-=item new()
-
-create an object. Do not use it directly, instead use:
-
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new();
     my $TicketColumnFilterObject = $Kernel::OM->Get('Kernel::System::Ticket::ColumnFilter');
 
 =cut
@@ -54,7 +48,7 @@ sub new {
     return $Self;
 }
 
-=item StateFilterValuesGet()
+=head2 StateFilterValuesGet()
 
 get a list of states within the given ticket is list
 
@@ -119,7 +113,7 @@ sub StateFilterValuesGet {
     return \%Data;
 }
 
-=item QueueFilterValuesGet()
+=head2 QueueFilterValuesGet()
 
 get a list of queues within the given ticket is list
 
@@ -186,7 +180,7 @@ sub QueueFilterValuesGet {
     return \%Data;
 }
 
-=item PriorityFilterValuesGet()
+=head2 PriorityFilterValuesGet()
 
 get a list of priorities within the given ticket is list
 
@@ -251,7 +245,7 @@ sub PriorityFilterValuesGet {
     return \%Data;
 }
 
-=item TypeFilterValuesGet()
+=head2 TypeFilterValuesGet()
 
 get a list of ticket types within the given ticket is list
 
@@ -316,7 +310,7 @@ sub TypeFilterValuesGet {
     return \%Data;
 }
 
-=item LockFilterValuesGet()
+=head2 LockFilterValuesGet()
 
 get a list of ticket lock values within the given ticket is list
 
@@ -382,7 +376,7 @@ sub LockFilterValuesGet {
     return \%Data;
 }
 
-=item ServiceFilterValuesGet()
+=head2 ServiceFilterValuesGet()
 
 get a list of services within the given ticket is list
 
@@ -447,7 +441,7 @@ sub ServiceFilterValuesGet {
     return \%Data;
 }
 
-=item SLAFilterValuesGet()
+=head2 SLAFilterValuesGet()
 
 get a list of service level agreements within the given ticket is list
 
@@ -512,7 +506,7 @@ sub SLAFilterValuesGet {
     return \%Data;
 }
 
-=item CustomerFilterValuesGet()
+=head2 CustomerFilterValuesGet()
 
 get a list of customer ids within the given ticket is list
 
@@ -581,7 +575,7 @@ sub CustomerFilterValuesGet {
     return \%Data;
 }
 
-=item CustomerUserIDFilterValuesGet()
+=head2 CustomerUserIDFilterValuesGet()
 
 get a list of customer users within the given ticket is list
 
@@ -650,7 +644,7 @@ sub CustomerUserIDFilterValuesGet {
     return \%Data;
 }
 
-=item OwnerFilterValuesGet()
+=head2 OwnerFilterValuesGet()
 
 get a list of ticket owners within the given ticket is list
 
@@ -730,7 +724,7 @@ sub OwnerFilterValuesGet {
                 UserID => $UserID,
             );
             if (%User) {
-                $Data{$UserID} = $User{UserFirstname} . ' ' . $User{UserLastname};
+                $Data{$UserID} = $User{UserFullname};
             }
         }
     }
@@ -738,7 +732,7 @@ sub OwnerFilterValuesGet {
     return \%Data;
 }
 
-=item ResponsibleFilterValuesGet()
+=head2 ResponsibleFilterValuesGet()
 
 get a list of agents responsible for the tickets within the given ticket list
 
@@ -818,7 +812,7 @@ sub ResponsibleFilterValuesGet {
                 UserID => $UserID,
             );
             if (%User) {
-                $Data{$UserID} = $User{UserFirstname} . ' ' . $User{UserLastname};
+                $Data{$UserID} = $User{UserFullname};
             }
         }
     }
@@ -826,7 +820,7 @@ sub ResponsibleFilterValuesGet {
     return \%Data;
 }
 
-=item DynamicFieldFilterValuesGet()
+=head2 DynamicFieldFilterValuesGet()
 
 get a list of a specific ticket dynamic field values within the given tickets list
 
@@ -926,7 +920,7 @@ sub DynamicFieldFilterValuesGet {
 
 =begin Internal:
 
-=item _GeneralDataGet()
+=head2 _GeneralDataGet()
 
 get data list
 
@@ -960,7 +954,7 @@ sub _GeneralDataGet {
         }
     }
 
-    my $FuctionName = $Param{FunctionName};
+    my $FunctionName = $Param{FunctionName};
 
     # set the backend file
     my $BackendModule = $Param{ModuleName};
@@ -994,7 +988,7 @@ sub _GeneralDataGet {
     }
 
     # get data list
-    my %DataList = $BackendObject->$FuctionName(
+    my %DataList = $BackendObject->$FunctionName(
         Valid  => 1,
         UserID => $Param{UserID},
     );
@@ -1057,14 +1051,12 @@ sub _TicketIDStringGet {
 
 =end Internal:
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
