@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -510,8 +510,7 @@ sub TicketSearch {
             $TicketHistoryJoins{$THRef} = 1;
             $SQLFrom .= sprintf
                 'INNER JOIN ticket_history %s ON st.id = %s.ticket_id ',
-                $THRef, $THRef,
-                ;
+                $THRef, $THRef;
         }
     }
 
@@ -1538,7 +1537,11 @@ sub TicketSearch {
     # catch searches for non-existing dynamic fields
     PARAMS:
     for my $Key ( sort keys %Param ) {
-        next PARAMS if !$Param{$Key};
+
+        # Only look at fields which start with DynamicField_ and contain a substructure that is meant for searching.
+        #   It could happen that similar scalar parameters are sent to this method, that should be ignored
+        #   (see bug#13412).
+        next PARAMS if !ref $Param{$Key};
         next PARAMS if $Key !~ /^DynamicField_(.*)$/;
 
         my $DynamicFieldName = $1;
@@ -2103,7 +2106,7 @@ sub TicketSearch {
                 ( join ', ', sort @List ),
                 $THRef,
                 $DBObject->Quote( $Param{TicketCloseTimeOlderDate} )
-                )
+            );
         }
     }
 
@@ -2165,7 +2168,7 @@ sub TicketSearch {
                 ( join ', ', sort @List ),
                 $THRef,
                 $DBObject->Quote( $Param{TicketCloseTimeNewerDate} )
-                )
+            );
         }
     }
 
@@ -2414,9 +2417,35 @@ sub TicketSearch {
                     $SQLExt .= " u.first_name $OrderBySuffix, u.last_name ";
                 }
             }
+            elsif (
+                $SortByArray[$Count] eq 'EscalationUpdateTime'
+                || $SortByArray[$Count] eq 'EscalationResponseTime'
+                || $SortByArray[$Count] eq 'EscalationSolutionTime'
+                || $SortByArray[$Count] eq 'EscalationTime'
+                || $SortByArray[$Count] eq 'PendingTime'
+                )
+            {
+
+                # Tickets with no Escalation or Pending time have '0' as value in the according ticket columns.
+                # When sorting by these columns always place ticket's with '0' value on the end, no matter order by.
+                if ( $Kernel::OM->Get('Kernel::System::DB')->{'DB::Type'} eq 'mysql' ) {
+
+                    # For MySQL create SQL order by query 'ORDER BY column_value = 0, column_value ASC/DESC'.
+                    $SQLSelect .= ', ' . $SortOptions{ $SortByArray[$Count] };
+                    $SQLExt
+                        .= ' ' . $SortOptions{ $SortByArray[$Count] };
+                }
+                else {
+
+                    # For PostgreSQL and Oracle transform selected 0 values to NULL and use 'NULLS LAST'
+                    #   in the end of SQL query.
+                    $SQLSelect .= ', ' . $SortOptions{ $SortByArray[$Count] } . ' AS order_value ';
+                    $SQLExt .= ' order_value ';
+                }
+            }
             else {
 
-                # regular sort
+                # Regular sort.
                 $SQLSelect .= ', ' . $SortOptions{ $SortByArray[$Count] };
                 $SQLExt    .= ' ' . $SortOptions{ $SortByArray[$Count] };
             }
@@ -2517,6 +2546,128 @@ sub TicketSearch {
     }
 }
 
+=head2 TicketCountByAttribute()
+
+Returns count of tickets per value for a specific attribute.
+
+    my $TicketCount = $TicketObject->TicketCountByAttribute(
+        Attribute => 'ServiceID',
+        TicketIDs => [ 1, 2, 3 ],
+    );
+
+Returns:
+
+    $TicketCount = {
+        Attribute_Value_1 => 1,
+        Attribute_Value_2 => 3,
+        ...
+    };
+
+=cut
+
+sub TicketCountByAttribute {
+    my ( $Self, %Param ) = @_;
+
+    if ( !$Param{Attribute} ) {
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'error',
+            Message  => 'Need Attribute!',
+        );
+        return;
+    }
+
+    # Check supported attributes.
+    my $Attribute           = $Param{Attribute};
+    my %AttributeToDatabase = (
+        Lock       => 'ticket_lock_id',
+        LockID     => 'ticket_lock_id',
+        Queue      => 'queue_id',
+        QueueID    => 'queue_id',
+        Priority   => 'ticket_priority_id',
+        PriorityID => 'ticket_priority_id',
+        Service    => 'service_id',
+        ServiceID  => 'service_id',
+        SLA        => 'sla_id',
+        SLAID      => 'sla_id',
+        State      => 'ticket_state_id',
+        StateID    => 'ticket_state_id',
+        Type       => 'type_id',
+        TypeID     => 'type_id',
+    );
+    if ( !$AttributeToDatabase{$Attribute} ) {
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'error',
+            Message  => "No matching database colum found for Attribute '$Attribute'!",
+        );
+        return;
+    }
+    my $DatabaseColumn = $AttributeToDatabase{$Attribute};
+
+    # Nothing to do.
+    return {} if !IsArrayRefWithData( $Param{TicketIDs} );
+    my @BindTicketIDs = map { \$_ } @{ $Param{TicketIDs} };
+
+    # Prepare value-type attributes.
+    my %AttributeValueLookup;
+    if ( $Attribute eq 'Lock' ) {
+        %AttributeValueLookup = $Kernel::OM->Get('Kernel::System::Lock')->LockList( UserID => 1 );
+    }
+    elsif ( $Attribute eq 'Queue' ) {
+        %AttributeValueLookup = $Kernel::OM->Get('Kernel::System::Queue')->QueueList( Valid => 0 );
+    }
+    elsif ( $Attribute eq 'Priority' ) {
+        %AttributeValueLookup = $Kernel::OM->Get('Kernel::System::Priority')->PriorityList( Valid => 0 );
+    }
+    elsif ( $Attribute eq 'Service' ) {
+        %AttributeValueLookup = $Kernel::OM->Get('Kernel::System::Service')->ServiceList(
+            Valid  => 0,
+            UserID => 1,
+        );
+    }
+    elsif ( $Attribute eq 'SLA' ) {
+        %AttributeValueLookup = $Kernel::OM->Get('Kernel::System::SLA')->SLAList(
+            Valid  => 0,
+            UserID => 1,
+        );
+    }
+    elsif ( $Attribute eq 'State' ) {
+        %AttributeValueLookup = $Kernel::OM->Get('Kernel::System::State')->StateList(
+            Valid  => 0,
+            UserID => 1,
+        );
+    }
+    elsif ( $Attribute eq 'Type' ) {
+        %AttributeValueLookup = $Kernel::OM->Get('Kernel::System::Type')->TypeList( Valid => 0 );
+    }
+    my $AttributeType = %AttributeValueLookup ? 'Value' : 'ID';
+
+    # Get count from database.
+    my $TicketIDString = join ',', ('?') x scalar @BindTicketIDs;
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
+    return if !$DBObject->Prepare(
+        SQL =>
+            'SELECT COUNT(*), ' . $DatabaseColumn
+            . ' FROM ticket'
+            . ' WHERE id IN (' . $TicketIDString . ')'
+            . ' AND ' . $DatabaseColumn . ' IS NOT NULL'
+            . ' GROUP BY ' . $DatabaseColumn,
+        Bind  => \@BindTicketIDs,
+        Limit => 10_000,
+    );
+    my %AttributeCount;
+    while ( my @Row = $DBObject->FetchrowArray() ) {
+        $AttributeCount{ $Row[1] } = $Row[0];
+    }
+
+    # No conversion necessary.
+    return \%AttributeCount if $AttributeType eq 'ID';
+
+    # Convert database IDs to values, skip entries with unknown value lookup.
+    my %AttributeCountConverted = map { $AttributeValueLookup{$_} => $AttributeCount{$_} }
+        grep { $AttributeValueLookup{$_} } sort keys %AttributeCount;
+    return \%AttributeCountConverted;
+}
+
 =head1 PRIVATE INTERFACE
 
 =head2 _TicketHistoryReferenceForSearchArgument
